@@ -46,10 +46,18 @@
 #include <stdio.h>
 #include <signal.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
 #include <stdlib.h>
 #include <fcntl.h>
 
+#ifdef G_OS_WIN32
+#include <share.h>
+#endif
+
+#include <glib/gstdio.h>
+
 /* If you have a strange unix, you get odd hard coded limits */
 #ifndef PATH_MAX
 #  define PATH_MAX 1024
@@ -494,13 +502,17 @@ get_tmpdir (void)
 static char *
 get_lock_fname (void)
 {
-        return g_strconcat (get_tmpdir (), "/bonobo-activation-register.lock", NULL);
+        return g_build_filename (get_tmpdir (),
+                                 "bonobo-activation-register.lock",
+                                 NULL);
 }
 
 static char *
 get_ior_fname (void)
 {
-        return g_strconcat (get_tmpdir (), "/bonobo-activation-server-ior", NULL);
+        return g_build_filename (get_tmpdir (),
+                                 "bonobo-activation-server-ior",
+                                 NULL);
 }
 
 static void
@@ -516,8 +528,8 @@ wait_for_lock (void)
         nanosleep (&timewait, NULL);
 
 #else
-#warning You will have bad performance without usleep
-        sleep (1);
+#warning You might have bad performance without usleep
+        g_usleep (10000);
 #endif
         access ("bonobo-activation lock wait", 0);
 }
@@ -526,6 +538,7 @@ static void
 rloc_file_lock (const BonoboActivationBaseServiceRegistry *registry, 
                 gpointer                                   user_data)
 {
+#if defined (F_SETFD) && defined (FD_CLOEXEC) && defined (F_SETLKW)
 	char *fn;
 	struct flock lock;
         int retval;
@@ -564,12 +577,30 @@ rloc_file_lock (const BonoboActivationBaseServiceRegistry *registry,
 	}
 
         g_free (fn);
+#elif defined (G_OS_WIN32)
+	char *fn = get_lock_fname ();
+        wchar_t *wfn = g_utf8_to_utf16 (fn, -1, NULL, NULL, NULL);
+        
+	while ((lock_fd = _wsopen (wfn, O_CREAT|O_RDWR|_O_SHORT_LIVED|_O_NOINHERIT, _SH_DENYRW, 0700)) < 0) {
+                if (errno == EACCES) {
+                        wait_for_lock ();
+		} else {
+                        g_message ("%s locking '%s'", g_strerror (errno), fn);
+			break;
+                }
+	}
+        g_free (wfn);
+        g_free (fn);
+#else
+        g_warning ("No locking implemented\n");
+#endif
 }
 
 static void
 rloc_file_unlock (const BonoboActivationBaseServiceRegistry *registry, 
                   gpointer                                   user_data)
 {
+#if defined (F_SETFD) && defined (FD_CLOEXEC) && defined (F_SETLKW)
         struct flock lock;
 
 	if (lock_fd >= 0) {
@@ -584,6 +615,12 @@ rloc_file_unlock (const BonoboActivationBaseServiceRegistry *registry,
 		close (lock_fd);
 		lock_fd = -1;
 	}
+#elif defined (G_OS_WIN32)
+	if (lock_fd >= 0) {
+		close (lock_fd);
+		lock_fd = -1;
+	}
+#endif
 }
 
 static char *
@@ -595,7 +632,7 @@ rloc_file_check (const BonoboActivationBaseServiceRegistry *registry,
 	char *fn;
 
         fn = get_ior_fname ();
-	fh = fopen (fn, "r");
+	fh = g_fopen (fn, "r");
         g_free (fn);
 
 	if (fh != NULL) {
@@ -627,7 +664,7 @@ rloc_file_register (const BonoboActivationBaseServiceRegistry *registry, const c
 	FILE *fh;
 
         fn = get_ior_fname ();
-	fh = fopen (fn, "w");
+	fh = g_fopen (fn, "w");
 
         if (fh != NULL) {
                 fprintf (fh, "%s\n", ior);
@@ -645,7 +682,7 @@ rloc_file_unregister (const BonoboActivationBaseServiceRegistry *registry,
 {
 	char *fn;
 
-	unlink ((fn = get_ior_fname ()));
+	g_unlink ((fn = get_ior_fname ()));
         g_free (fn);
 }
 
@@ -657,8 +694,6 @@ static const BonoboActivationBaseServiceRegistry rloc_file = {
 	rloc_file_unregister
 };
 
-#define STRMATCH(x, y) ((!x && !y) || (x && y && !strcmp(x, y)))
-
 static CORBA_Object
 local_re_check_fn (const Bonobo_ActivationEnvironment *environment,
                    const char                         *act_iid,
@@ -693,9 +728,24 @@ void
 bonobo_activation_base_service_init (void)
 {
         const char *override_cmd;
+#ifndef G_OS_WIN32
         static const char *bonobo_activation_ac_cmd[] =
                 { SERVER_LIBEXECDIR "/bonobo-activation-server",
                   "--ac-activate", "--ior-output-fd=%d", NULL };
+#else
+        static const char *bonobo_activation_ac_cmd[] =
+                { NULL,
+                  "--ac-activate", "--ior-output-fd=%d", NULL };
+        static gboolean beenhere = FALSE;
+
+        if (!beenhere) {
+                bonobo_activation_ac_cmd[0] =
+                        g_build_filename (_bonobo_activation_win32_get_server_libexecdir (),
+                                          "bonobo-activation-server",
+                                          NULL);
+                beenhere = TRUE;
+        }
+#endif
 
 	bonobo_activation_base_service_activator_add (local_activator, 0);
 
