@@ -35,17 +35,12 @@
 #include "bonobo-activation-register.h"
 #include "bonobo-activation-version.h"
 #include <sys/types.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 #include <fcntl.h>
 #include <glib.h>
-#include <netdb.h>
 #include <popt.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/socket.h>
-#include <sys/wait.h>
 #include <unistd.h>
 
 /****************** ORBit-specific stuff ****************/
@@ -60,6 +55,162 @@ static gboolean is_initialized = FALSE;
 /* prevent registering with OAF when bonobo_activation_active_server_register() */
 gboolean bonobo_activation_private = FALSE;
 
+#ifdef G_OS_WIN32
+
+#ifndef PIC
+#error Must build as a DLL
+#endif
+
+#include <windows.h>
+#include <mbstring.h>
+
+static const char *prefix;
+static const char *server_libexecdir;
+static const char *serverinfodir;
+static const char *server_confdir;
+static const char *localedir;
+
+static HMODULE hmodule;
+
+char *
+_bonobo_activation_win32_replace_prefix (const char *runtime_prefix,
+                                         const char *configure_time_path)
+{
+  if (strncmp (configure_time_path, PREFIX "/", strlen (PREFIX) + 1) == 0) {
+          return g_strconcat (runtime_prefix,
+                              configure_time_path + strlen (PREFIX),
+                              NULL);
+  } else
+          return g_strdup (configure_time_path);
+}
+
+/* DllMain that just stores the DLL's hmodule */
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL,
+	 DWORD     fdwReason,
+	 LPVOID    lpvReserved)
+{
+        switch (fdwReason) {
+        case DLL_PROCESS_ATTACH:
+                hmodule = hinstDLL;
+                break;
+        }
+        return TRUE;
+}
+
+G_LOCK_DEFINE_STATIC (mutex);
+
+static void
+setup_runtime_paths (void)
+{
+        char cpbfr[1000];
+        wchar_t wcbfr[1000];
+        char *full_prefix = NULL;
+        char *cp_prefix = NULL;
+  
+        G_LOCK (mutex);
+        if (prefix != NULL) {
+                G_UNLOCK (mutex);
+                return;
+        }
+
+        if (G_WIN32_HAVE_WIDECHAR_API ()) {
+                /* NT-based Windows */
+                if (GetModuleFileNameW (hmodule, wcbfr, G_N_ELEMENTS (wcbfr))) {
+                        full_prefix = g_utf16_to_utf8 (wcbfr, -1,
+                                                   NULL, NULL, NULL);
+                        if (GetShortPathNameW (wcbfr, wcbfr, G_N_ELEMENTS (wcbfr)) &&
+                            WideCharToMultiByte (CP_ACP, 0, wcbfr, -1,
+                                                 cpbfr, G_N_ELEMENTS (cpbfr),
+                                                 NULL, NULL))
+                                cp_prefix = g_strdup (cpbfr);
+                        else if (full_prefix)
+                                cp_prefix = g_locale_from_utf8 (full_prefix, -1,
+                                                                NULL, NULL, NULL);
+                }
+        } else {
+                /* Win9x */
+                if (GetModuleFileNameA (hmodule, cpbfr, G_N_ELEMENTS (cpbfr))) {
+                        full_prefix = g_locale_to_utf8 (cpbfr, -1,
+                                                        NULL, NULL, NULL);
+                        cp_prefix = g_strdup (cpbfr);
+                }
+        }
+
+        if (full_prefix != NULL) {
+                gchar *p = strrchr (full_prefix, '\\');
+                if (p != NULL)
+                        *p = '\0';
+      
+                p = strrchr (full_prefix, '\\');
+                if (p && (g_ascii_strcasecmp (p + 1, "bin") == 0))
+                        *p = '\0';
+                prefix = full_prefix;
+        }
+                  
+        if (cp_prefix != NULL) {
+                gchar *p = _mbsrchr (cp_prefix, '\\');
+                if (p != NULL)
+                        *p = '\0';
+      
+                p = _mbsrchr (cp_prefix, '\\');
+                if (p && (g_ascii_strcasecmp (p + 1, "bin") == 0))
+                        *p = '\0';
+                
+        }
+
+        server_libexecdir = _bonobo_activation_win32_replace_prefix (full_prefix, SERVER_LIBEXECDIR);
+        serverinfodir = _bonobo_activation_win32_replace_prefix (full_prefix, SERVERINFODIR);
+        server_confdir = _bonobo_activation_win32_replace_prefix (full_prefix, SERVER_CONFDIR);
+        localedir = _bonobo_activation_win32_replace_prefix (cp_prefix, BONOBO_ACTIVATION_LOCALEDIR);
+
+        G_UNLOCK (mutex);
+}
+
+const char *
+_bonobo_activation_win32_get_prefix (void)
+{
+        setup_runtime_paths ();
+
+        return prefix;
+}
+
+const char *
+_bonobo_activation_win32_get_server_libexecdir (void)
+{
+        setup_runtime_paths ();
+
+        return server_libexecdir;
+}
+
+const char *
+_bonobo_activation_win32_get_serverinfodir (void)
+{
+        setup_runtime_paths ();
+
+        return serverinfodir;
+}
+
+const char *
+_bonobo_activation_win32_get_server_confdir (void)
+{
+        setup_runtime_paths ();
+
+        return server_confdir;
+}
+
+const char *
+_bonobo_activation_win32_get_localedir (void)
+{
+        setup_runtime_paths ();
+
+        return localedir;
+}
+
+#undef BONOBO_ACTIVATION_LOCALEDIR
+#define BONOBO_ACTIVATION_LOCALEDIR _bonobo_activation_win32_get_localedir ()
+
+#endif
 
 /**
  * bonobo_activation_orb_get:
@@ -205,6 +356,31 @@ struct poptOption bonobo_activation_popt_options[] = {
         { NULL }
 };
 
+static const GOptionEntry bonobo_activation_goption_options[] = {
+        { "oaf-ior-fd", '\0', 0,  G_OPTION_ARG_INT, &bonobo_activation_ior_fd, 
+          N_("File descriptor to print IOR on"), N_("FD") },
+        { "oaf-activate-iid", '\0', 0, G_OPTION_ARG_STRING, &bonobo_activation_activate_iid, 
+          N_("IID to activate"), "IID" },
+        { "oaf-private", '\0', 0, G_OPTION_ARG_NONE, &bonobo_activation_private, 
+          N_("Prevent registering of server with OAF"), NULL },
+        { NULL }
+};
+
+GOptionGroup *
+bonobo_activation_get_goption_group (void)
+{
+	GOptionGroup *group;
+
+	group = g_option_group_new ("bonbo-activation",
+				    N_("Bonobo Activation"),
+				    N_("Show Bonobo Activation options"),
+				    NULL, NULL);
+	g_option_group_set_translation_domain (group, GETTEXT_PACKAGE);
+	g_option_group_add_entries (group, bonobo_activation_goption_options);
+
+	return group;
+}
+
 /**
  * bonobo_activation_activation_iid_get:
  *
@@ -237,9 +413,10 @@ bonobo_activation_postinit (gpointer app, gpointer mod_info)
 {
 	bonobo_activation_base_service_init ();
 
+#if defined (F_SETFD) && defined (FD_CLOEXEC)
 	if (bonobo_activation_ior_fd > 2)
 		fcntl (bonobo_activation_ior_fd, F_SETFD, FD_CLOEXEC);
-
+#endif
         if (bonobo_activation_activate_iid)
                 g_timeout_add_full (G_PRIORITY_LOW,
                                     BONOBO_ACTIVATION_FACTORY_TIMEOUT,
@@ -257,6 +434,10 @@ do_barrier (int signum)
 {
 	volatile int barrier = 1;
 
+#ifndef HAVE_SIGACTION
+        signal (signum, do_barrier);
+#endif
+
 	while (barrier);
 }
 #endif
@@ -356,10 +537,16 @@ bonobo_activation_orb_init (int *argc, char **argv)
         CORBA_Context def_ctx;
 	CORBA_Environment ev;
 	const char *hostname;
+	const gchar *orb_id;
 
 	CORBA_exception_init (&ev);
 
-	bonobo_activation_orb = CORBA_ORB_init (argc, argv, "orbit-local-mt-orb", &ev);
+#ifdef HAVE_GTHREADS
+	orb_id = "orbit-local-mt-orb";
+#else
+	orb_id = "orbit-local-non-threaded-orb";
+#endif
+	bonobo_activation_orb = CORBA_ORB_init (argc, argv, orb_id, &ev);
 	g_assert (ev._major == CORBA_NO_EXCEPTION);
 
 	bonobo_activation_init_activation_env ();
@@ -381,10 +568,17 @@ bonobo_activation_orb_init (int *argc, char **argv)
 
 #ifdef BONOBO_ACTIVATION_DEBUG
 	if (getenv ("BONOBO_ACTIVATION_TRAP_SEGV")) {
+#ifdef HAVE_SIGACTION
 		struct sigaction sa;
 		sa.sa_handler = do_barrier;
 		sigaction (SIGSEGV, &sa, NULL);
 		sigaction (SIGPIPE, &sa, NULL);
+#else
+                signal (SIGSEGV, do_barrier);
+#ifdef SIGPIPE
+                signal (SIGPIPE, do_barrier);
+#endif
+#endif
 	}
 	if (getenv ("BONOBO_ACTIVATION_BARRIER_INIT")) {
 		volatile int barrier = 1;
