@@ -237,6 +237,7 @@ bonobo_object_finalize_servant (PortableServer_Servant servant,
 				CORBA_Environment *ev)
 {
 	BonoboObject *object = bonobo_object(servant);
+	BonoboObjectClass *klass = BONOBO_OBJECT_GET_CLASS(object);
 
 #ifdef G_ENABLE_DEBUG
 	if(_bonobo_debug_flags & BONOBO_DEBUG_LIFECYCLE)
@@ -245,6 +246,11 @@ bonobo_object_finalize_servant (PortableServer_Servant servant,
 				    object);
 #endif /* G_ENABLE_DEBUG */
 
+	if (klass->poa_fini_fn)
+		klass->poa_fini_fn (servant, ev);
+	else /* Actually quicker and nicer */
+		PortableServer_ServantBase__fini (servant, ev);
+
 	g_object_unref (G_OBJECT (object));
 }
 
@@ -615,13 +621,10 @@ bonobo_object_query_local_interface (BonoboObject *object,
 				     const char   *repo_id)
 {
 	GList             *l;
-	CORBA_Object       corba_retval = CORBA_OBJECT_NIL;
 	CORBA_Environment  ev;
 
 	g_return_val_if_fail (BONOBO_IS_OBJECT (object), NULL);
 
-	corba_retval = CORBA_OBJECT_NIL;
-
 	CORBA_exception_init (&ev);
 
 	for (l = object->priv->ao->objs; l; l = l->next){
