@@ -46,11 +46,8 @@ static void
 corba_object_proxy_value_free (GValue *value)
 {
 	if (value->data[0].v_pointer) {
-		CorbaObjectProxy *proxy;
 		CORBA_Environment ev;
 
-		proxy = corba_object_proxy_get (G_VALUE_TYPE (value));
-		
 		CORBA_exception_init (&ev);
 		CORBA_Object_release (value->data[0].v_pointer, &ev);
 		CORBA_exception_free (&ev);
@@ -62,10 +59,6 @@ corba_object_proxy_value_copy (const GValue *src_value,
 			       GValue       *dest_value)
 {
 	if (src_value->data[0].v_pointer) {
-		CorbaObjectProxy *proxy;
-
-		proxy = corba_object_proxy_get (G_VALUE_TYPE (src_value));
-		
 		dest_value->data[0].v_pointer = CORBA_Object_duplicate (
 			src_value->data[0].v_pointer, NULL);
 	} else
@@ -123,10 +116,6 @@ corba_object_proxy_lcopy_value (const GValue *value,
 	else if (collect_flags & G_VALUE_NOCOPY_CONTENTS)
 		*corba_p = value->data[0].v_pointer;
 	else {
-		CorbaObjectProxy *proxy;
-
-		proxy = corba_object_proxy_get (G_VALUE_TYPE (value));;
-
 		*corba_p = CORBA_Object_duplicate (value->data[0].v_pointer, NULL);
 	}
 
@@ -192,8 +181,8 @@ bonobo_ ## name ## _get_type (void)						\
 	return type;								\
 }
 
-BONOBO_TYPE_CORBA_OBJECT_IMPL (corba_object, "CorbaObject", TC_CORBA_Object, FALSE);
-BONOBO_TYPE_CORBA_OBJECT_IMPL (unknown, "BonoboUnknown", TC_Bonobo_Unknown, TRUE);
+BONOBO_TYPE_CORBA_OBJECT_IMPL (corba_object, "CorbaObject", TC_CORBA_Object, FALSE)
+BONOBO_TYPE_CORBA_OBJECT_IMPL (unknown, "BonoboUnknown", TC_Bonobo_Unknown, TRUE)
 
 static gpointer
 corba_any_copy (gpointer any)
@@ -429,15 +418,19 @@ bonobo_closure_invoke_va_list (GClosure *closure,
  * @closure: a standard GClosure
  * @return_type: the type of the first va_arg argument in a
  * set of type / arg pairs.
+ * @Varargs: list of type / arg pairs
  *
  * Invokes the closure with the arguments.
  *
- * Example:
- *
+ * Examples:
+ * <informalexample><programlisting>
  *    bonobo_closure_invoke (closure, G_TYPE_NONE, G_TYPE_INT, first_arg, 0);
+ * </programlisting></informalexample>
  *
+ * <informalexample><programlisting>
  *    glong retval;
  *    bonobo_closure_invoke (closure, G_TYPE_LONG, & retval, 0);
+ * </programlisting></informalexample>
  *
  **/
 void
@@ -506,6 +499,8 @@ bonobo_closure_invoke (GClosure *closure,
  * 
  * Does the necessary refcounting magic and returns a directly
  * storable closure
+ *
+ * Returns: @closure.
  **/
 GClosure *
 bonobo_closure_store (GClosure        *closure,
